#
#
#

require 'runit/testcase'
require 'runit/cui/testrunner'

require '/home/akira/src/me/doc/ohmsha/rubyapp/mylock.rb'

class TestMyLock < RUNIT::TestCase
  def setup
    @testdir  = '/tmp/mylock_test'
    @lockdir  = 'lockdir'
    @lockfile = 'lockfile'

    @timeout  = 5
    @maxtry   = 2

    Dir.chdir(@testdir)
    Dir.mkdir(@lockdir)
    File.open(File.join(@lockdir, @lockfile), 'w') {}
  end

  def teardown
    Dir.chdir(@testdir)
    Dir.foreach(@lockdir) do |f|
      if /^#{Regexp.quote(@lockfile)}(\d*)$/o =~ f
	File.unlink(File.join(@lockdir, f))
      end
    end
    Dir.rmdir(@lockdir)
  end

  def test_lock
    l  = MyLock.new(@lockdir, @lockfile, @timeout, @maxtry)
    l2 = MyLock.new(@lockdir, @lockfile, @timeout, @maxtry)
    l3 = MyLock.new(@lockdir, @lockfile, @timeout, @maxtry)
    assert_instance_of(MyLock, l)

    assert(l.lock)
    assert(l.unlock)
    assert_equal("OK", l.lock{"OK"})
    assert_equal("OK", l.lock{sleep(@timeout + 1); "OK"})

    l.lock do
      assert_exception(MyLock::LockError) {l2.lock}
    end

    begin
      l.lock {raise RuntimeError}
    rescue RuntimeError
    end
    assert_equal("OK", l.lock{"OK"})

    l.lock
    sleep(@timeout + 1)
    assert(l2.lock)
    assert_exception(MyLock::UnLockError) {l.unlock}
    assert(l2.unlock)
    assert_exception(MyLock::UnLockError) {l.unlock}

    l.lock
    sleep(@timeout + 1)
    assert(l2.lock)
    assert_exception(MyLock::LockError) {l3.lock}
    assert(l2.unlock)

    require 'thread'
    l.lock
    sleep(@timeout + 1)
    ths = []
    30.times do
      x = MyLock.new(@lockdir, @lockfile, @timeout, @maxtry)
      ths << Thread.new do
	begin
	  x.lock {sleep(@timeout - 1); 'OK'}
	rescue MyLock::LockError
	end
      end
    end
    ths.join
    assert_equal(1, ths.collect {|th| th.value}.compact.size)
  end

  def test_s_lock
    l = MyLock.lock(@lockdir, @lockfile, @timeout, @maxtry)
    assert_instance_of(MyLock, l)
    assert(l.unlock)

    l = MyLock.lock(@lockdir, @lockfile, @timeout, @maxtry)
    assert_exception(MyLock::LockError) do
      MyLock.lock(@lockdir, @lockfile, @timeout, @maxtry)
    end
    l.unlock

    MyLock.lock(@lockdir, @lockfile, @timeout, @maxtry)
    sleep(@timeout + 1)
    assert_equal('OK',
		 MyLock.lock(@lockdir, @lockfile, @timeout, @maxtry) {'OK'})
  end
end

if $0 == __FILE__
  if ARGV.size == 0
    suite = TestMyLock.suite
  else
    suite = RUNIT::TestSuite.new
    ARGV.each do |testmethod|
      suite.add_test(TestMyLock.new(testmethod))
    end
  end
  RUNIT::CUI::TestRunner.run(suite)
end
